package com.example.workflow;

import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;

import javax.inject.Named;
import javax.mail.*;
import javax.mail.Flags.Flag;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMultipart;
import javax.mail.search.FlagTerm;
import java.io.IOException;
import java.util.Properties;

@Named
public class EmailService implements JavaDelegate {

    @Override
    public void execute(DelegateExecution delegateExecution) throws Exception {
        EmailContent emailContent = retrieveLatestEmail();
        parseEmailContent(emailContent.getText(), delegateExecution);
        //Issue topic is parsed from what the email subject is
        delegateExecution.setVariable("issueTopic", emailContent.getSubject());
        delegateExecution.setVariable("EmailAddress", emailContent.getFrom());
    }

    //Method here makes email message be parsed into the textfield's for ticket review
    //For example if text beings with "Issue Description: " any text beyond this would be parsed as the issue description
    private void parseEmailContent(String emailText, DelegateExecution execution) {
        String[] lines = emailText.split("\n");
        for (String line : lines) {
            if (line.startsWith("Issue Description:")) {
                execution.setVariable("issueDescription", line.split(":", 2)[1].trim());
            } else if (line.startsWith("Effect:")) {
                execution.setVariable("effect", line.split(":", 2)[1].trim());
            } else if (line.startsWith("Priority:")) {
                execution.setVariable("priority", line.split(":", 2)[1].trim());
            } else if (line.startsWith("Username:")) {
                execution.setVariable("userName", line.split(":", 2)[1].trim()); // Set username variable
            }
        }
    }

    //EmailService Configurations
    private EmailContent retrieveLatestEmail() throws MessagingException, IOException {
        String host = "imap.gmail.com";
        String storeType = "imaps";
        String user = "user.ticketing.systemg2@gmail.com";
        String password = "xpzjeayahydrjiib";

        Properties properties = new Properties();
        properties.put("mail.imap.host", host);
        properties.put("mail.imap.port", "993");
        properties.put("mail.imap.starttls.enable", "true");
        properties.put("mail.imap.ssl.trust", host);
        Session emailSession = Session.getDefaultInstance(properties);

        Store store = emailSession.getStore(storeType);
        store.connect(host, user, password);

        Folder inbox = store.getFolder("Inbox");
        inbox.open(Folder.READ_WRITE);

        Message[] messages = inbox.search(new FlagTerm(new Flags(Flag.SEEN), false));
        if (messages.length == 0) return new EmailContent("", "", ""); // No new messages

        Message message = messages[messages.length - 1];
        message.setFlag(Flag.SEEN, true);

        String subject = message.getSubject();
        String text = getTextFromMessage(message);
        String from = "";

        // Extract only the email address, not the name
        Address[] fromAddresses = message.getFrom();
        if (fromAddresses.length > 0 && fromAddresses[0] instanceof InternetAddress) {
            InternetAddress internetAddress = (InternetAddress) fromAddresses[0];
            from = internetAddress.getAddress(); // This will only get the email address
        }

        inbox.close(false);
        store.close();

        return new EmailContent(text, subject, from);
    }


    private String getTextFromMessage(Message message) throws MessagingException, IOException {
        if (message.isMimeType("text/plain")) {
            return message.getContent().toString();
        } else if (message.isMimeType("multipart/*")) {
            MimeMultipart mimeMultipart = (MimeMultipart) message.getContent();
            return getTextFromMimeMultipart(mimeMultipart);
        }
        return "";
    }

    private String getTextFromMimeMultipart(MimeMultipart mimeMultipart) throws MessagingException, IOException {
        String result = "";
        for (int i = 0; i < mimeMultipart.getCount(); i++) {
            BodyPart bodyPart = mimeMultipart.getBodyPart(i);
            if (bodyPart.isMimeType("text/plain")) {
                result += "\n" + bodyPart.getContent();
            } else if (bodyPart.getContent() instanceof MimeMultipart) {
                result += getTextFromMimeMultipart((MimeMultipart)bodyPart.getContent());
            }
        }
        return result;
    }

    private class EmailContent {
        private final String text;
        private final String subject;
        private final String from; // New field to store the sender's email

        public EmailContent(String text, String subject, String from) {
            this.text = text;
            this.subject = subject;
            this.from = from;
        }

        public String getText() {
            return text;
        }

        public String getSubject() {
            return subject;
        }

        public String getFrom() {
            return from;
        }
    }
}
