package com.example.workflow;

import jakarta.inject.Named;
import org.camunda.bpm.engine.delegate.DelegateExecution;
import org.camunda.bpm.engine.delegate.JavaDelegate;
import org.springframework.jdbc.core.JdbcTemplate;

@Named
public class InsertTicketDataDelegate implements JavaDelegate {

    private JdbcTemplate jdbcTemplate;

    public InsertTicketDataDelegate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    @Override
    public void execute(DelegateExecution execution) throws Exception {
        // Extracted form field values using their keys and converts them into variables
        String username = (String) execution.getVariable("userName");
        String email = (String) execution.getVariable("EmailAddress");
        String issueTopic = (String) execution.getVariable("issueTopic");
        String issueDescription = (String) execution.getVariable("issueDescription");
        String effect = (String) execution.getVariable("effect");
        String priority = (String) execution.getVariable("priority");
        String estimatedTime = (String) execution.getVariable("estimatedTime");

        //Will populate already existing table called ticket_submission that I created in h2-console using a schema script
        String sql = "INSERT INTO ticket_submission (username, email, issue_topic, issue_description, effect, priority, estimatedTime, status) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";

        int rowsAffected = jdbcTemplate.update(sql, username, email, issueTopic, issueDescription, effect, priority, estimatedTime, "Open");
        //detects if any change was made to table and determines if ticket was saved if row increments
        if (rowsAffected > 0) {
            System.out.println("Ticket saved successfully.");
        } else {
            System.out.println("Email submission not received.");
        }
    }
}